############################################################
#Analysis of convergence in GDP per worker of EU27 regions #
############################################################

rm(list=ls(all=TRUE))
library(stargazer)

#Load dataset
load("datasetNUTS2_CE2010_EU27_forGRV_analysis")

#Define variable
  GDPpw <- samplePerWorkerGDP_27
  
  GDPpwRel <- samplePerWorkerGDPRel_27
  colnames(GDPpwRel) <- c(1991:2008)
  
  GDPpwGR <- samplePerWorkerGDPGR_27

  numYears <- ncol(GDPpw)

  #Firtst and last year of the sample
  PROD.InitialYear.REL <- (GDPpwRel[,1])  
  PROD.FinalYear.REL <-  (GDPpwRel[,(numYears)])

  #======================#
  # Convergence Analysis #
  #======================#
   
    LogGDPpwRel_initialYear <- log(GDPpwRel[,1])    
    
    #Average growth rate of each single regions
    #NB: use geometric mean!!      
    gm_mean = function(a){prod(a+1)^(1/length(a))-1}
    averageGrowthRateSingleRegions <- apply(GDPpwGR, 1, gm_mean)

    #***************************************************#
    # Conditional Convergence (Augmented Solow's Model) #
    #***************************************************#

	#Load Human Capital data Eurostat-Pose
	hc_EU.Pose.df <- read.csv("humanCapitalRegNUTS2.csv",header=TRUE,sep=",",na.strings="NA")
	hc_EU.Pose <- data.matrix(hc_EU.Pose.df)

	hc_EU.Pose_oursample <- hc_EU.Pose[,c(5:22)]
    mean_hc_index <- rowMeans(hc_EU.Pose_oursample)
	mean_invRate <- rowMeans(sampleInvestmentRate_27[,1:ncol(sampleInvestmentRate_27)])
    mean_empGr <- rowMeans(sampleTotalEmploymentGR_27) 
	mean_empGR_augm <- mean_empGr+0.05

	conditionalConvergence_linear <-lm(c(averageGrowthRateSingleRegions) ~ LogGDPpwRel_initialYear + +log(mean_invRate) +log(mean_empGR_augm)+log(mean_hc_index))
	summary(conditionalConvergence_linear)
	
	
	
	#Load and define instruments
	instruments <- read.csv("instruments.csv")
    
    #Possible instruments for mean_invRate
    INV.RATE.1991 <- instruments[,2]
    AV.REAL.INT.RATE <- instruments[,3]

    #Possible instruments for mean_empGr
    EMP.GR.1991 <- instruments[,4]+0.05
    POP.ACTIVE.on.TOTpop.1991 <- instruments[,5]
    UNEM.RATE.1991 <- instruments[,6]
    UNEM.RATE.1991[UNEM.RATE.1991==0] <- NA
    
    #Possible instruments for mean_hc_index        
    HC.INDEX.1991 <- hc_EU.Pose_oursample[,1]

	#---------------------------------------------#
	# Consider the investment rate as endogenous! #
	#---------------------------------------------#

    #****************************************************************************#
    # 2SLS with ONE endogenous (mean_invRate) and ONE instrument (INV.RATE.1991) #
    #****************************************************************************#
    #1) First-stage: regress invRate on all the exogenous+instruments
    firstStage_invRate <- lm(log(mean_invRate) ~ LogGDPpwRel_initialYear +log(mean_empGR_augm)+log(mean_hc_index)+ log(INV.RATE.1991))
    summary(firstStage_invRate)
    
    #2) Second-stage: regress y on all the exogenous+fitted values first stage
	fitted_firstStage <- firstStage_invRate$fitted
    conditionalConvergence_linear_endgInvRate <-lm(c(averageGrowthRateSingleRegions) ~ LogGDPpwRel_initialYear + fitted_firstStage +log(mean_empGR_augm)+log(mean_hc_index))
	summary(conditionalConvergence_linear_endgInvRate)

    #*********************************************************************************************	    
    # 2SLS with ONE endogenous (mean_invRate) and TWO instruments (INV.RATE.1991,AV.REAL.INT.RATE)
    #*********************************************************************************************	    
    #1) First-stage: regress invRate on all the exogenous+instruments
    firstStage_invRate <- lm(log(mean_invRate) ~ LogGDPpwRel_initialYear +log(mean_empGR_augm)+log(mean_hc_index)+ log(INV.RATE.1991)+log(AV.REAL.INT.RATE))
    summary(firstStage_invRate)
    
    library(car)
    linearHypothesis(firstStage_invRate, c("log(INV.RATE.1991)=0","log(AV.REAL.INT.RATE)=0"))
       
    #2) Second-stage: regress y on all the exogenous+fitted values first stage
	fitted_firstStage <- firstStage_invRate$fitted
    conditionalConvergence_linear_endgInvRate <-lm(c(averageGrowthRateSingleRegions) ~ LogGDPpwRel_initialYear + fitted_firstStage +log(mean_empGR_augm)+log(mean_hc_index))
	summary(conditionalConvergence_linear_endgInvRate)
 
    
	#----------------------------------------------------------------------------------#
	# Consider the investment rate, employment growth and human capital as endogenous! #
	#----------------------------------------------------------------------------------#

    #*********************************************************************************************	    
    # 2SLS with TRHEE endogenous (mean_invRate, mean_empGR_augm, mean_hc_index) and TRHEE instruments (INV.RATE.1991, EMP.GR.1991, HC.INDEX.1991)
    #*********************************************************************************************	    
    #1) First-stage: regress invRate on all the exogenous+instruments
    firstStage_invRate <- lm(log(mean_invRate) ~ LogGDPpwRel_initialYear + log(INV.RATE.1991) + log(EMP.GR.1991)+ log(HC.INDEX.1991), na.action="na.exclude")
    summary(firstStage_invRate)

    linearHypothesis(firstStage_invRate, c("log(INV.RATE.1991)=0","log(EMP.GR.1991)=0", "log(HC.INDEX.1991)=0"))

    firstStage_empGr <- lm(log(mean_empGR_augm) ~ LogGDPpwRel_initialYear + log(INV.RATE.1991) + log(EMP.GR.1991)+  log(HC.INDEX.1991), na.action="na.exclude")
    summary(firstStage_empGr)
    linearHypothesis(firstStage_invRate, c("log(INV.RATE.1991)=0","log(EMP.GR.1991)=0", "log(HC.INDEX.1991)=0"))

    firstStage_hc <- lm(log(mean_hc_index) ~ LogGDPpwRel_initialYear + log(INV.RATE.1991) + log(EMP.GR.1991)+  log(HC.INDEX.1991), na.action="na.exclude")
    summary(firstStage_hc)
    linearHypothesis(firstStage_invRate, c("log(INV.RATE.1991)=0","log(EMP.GR.1991)=0", "log(HC.INDEX.1991)=0"))

    
    #2) Second-stage: regress y on all the exogenous+fitted values first stage
	fitted_firstStage_invRate <- fitted(firstStage_invRate)
	fitted_firstStage_empGr <-  fitted(firstStage_empGr)
	fitted_firstStage_hc <- fitted( firstStage_hc)
	
	
    conditionalConvergence_linear_endgThree <-lm(c(averageGrowthRateSingleRegions) ~ LogGDPpwRel_initialYear + fitted_firstStage_invRate + fitted_firstStage_empGr + fitted_firstStage_hc)
	summary(conditionalConvergence_linear_endgThree)
	
    
    #*********************************************************************************************	    
    # 2SLS with TRHEE endogenous (mean_invRate, mean_empGR_augm, mean_hc_index) and SIX instruments (INV.RATE.1991, AV.REAL.INT.RATE, EMP.GR.1991, POP.ACTIVE.on.TOTpop.1991, UNEM.RATE.1991, HC.INDEX.1991)
    #*********************************************************************************************	    
    #1) First-stage: regress invRate on all the exogenous+instruments
    firstStage_invRate <- lm(log(mean_invRate) ~ LogGDPpwRel_initialYear + log(INV.RATE.1991) + log(AV.REAL.INT.RATE) +log(EMP.GR.1991)+ log(POP.ACTIVE.on.TOTpop.1991)+ log(UNEM.RATE.1991) +log(HC.INDEX.1991), na.action="na.exclude")
    summary(firstStage_invRate)

    linearHypothesis(firstStage_invRate, c("log(INV.RATE.1991)=0", "log(AV.REAL.INT.RATE)=0", "log(EMP.GR.1991)=0", "log(POP.ACTIVE.on.TOTpop.1991)=0", "log(UNEM.RATE.1991)=0", "log(HC.INDEX.1991)=0"))

    firstStage_empGr <- lm(log(mean_empGR_augm) ~ LogGDPpwRel_initialYear + log(INV.RATE.1991) + log(AV.REAL.INT.RATE) +log(EMP.GR.1991)+ log(POP.ACTIVE.on.TOTpop.1991)+ log(UNEM.RATE.1991) +log(HC.INDEX.1991), na.action="na.exclude")
    summary(firstStage_empGr)
    linearHypothesis(firstStage_empGr, c("log(INV.RATE.1991)=0", "log(AV.REAL.INT.RATE)=0", "log(EMP.GR.1991)=0", "log(POP.ACTIVE.on.TOTpop.1991)=0", "log(UNEM.RATE.1991)=0", "log(HC.INDEX.1991)=0"))

    firstStage_hc <- lm(log(mean_hc_index) ~LogGDPpwRel_initialYear + log(INV.RATE.1991) + log(AV.REAL.INT.RATE) +log(EMP.GR.1991)+ log(POP.ACTIVE.on.TOTpop.1991)+ log(UNEM.RATE.1991) +log(HC.INDEX.1991), na.action="na.exclude")
    summary(firstStage_hc)
    linearHypothesis(firstStage_hc, c("log(INV.RATE.1991)=0", "log(AV.REAL.INT.RATE)=0", "log(EMP.GR.1991)=0", "log(POP.ACTIVE.on.TOTpop.1991)=0", "log(UNEM.RATE.1991)=0", "log(HC.INDEX.1991)=0"))

    
    #2) Second-stage: regress y on all the exogenous+fitted values first stage
	fitted_firstStage_invRate <- fitted(firstStage_invRate)
	fitted_firstStage_empGr <-  fitted(firstStage_empGr)
	fitted_firstStage_hc <- fitted( firstStage_hc)
	
	
    conditionalConvergence_linear_endg <-lm(c(averageGrowthRateSingleRegions) ~ LogGDPpwRel_initialYear + fitted_firstStage_invRate + fitted_firstStage_empGr + fitted_firstStage_hc)
	summary(conditionalConvergence_linear_endg)
    
	stargazer(conditionalConvergence_linear, conditionalConvergence_linear_endgInvRate, conditionalConvergence_linear_endgThree, conditionalConvergence_linear_endg, type="text")

    #============================# 
    # SEMIPARAMETRIC MODEL -> CF #
    #============================# 
    library(mgcv)   
   
	 conditionalConvergence_gam <-gam(c(averageGrowthRateSingleRegions) ~ s(LogGDPpwRel_initialYear)  + s(log(mean_invRate)) +s(log(mean_empGR_augm))+s(log(mean_hc_index)))
	summary(conditionalConvergence_gam)
plot(conditionalConvergence_gam, page=1)

	#---------------------------------------------#
	# Consider the investment rate as endogenous! #
	#---------------------------------------------#

    #**************************************************************************#
    # CF with ONE endogenous (mean_invRate) and ONE instrument (INV.RATE.1991) #
    #**************************************************************************#
     #1) First-stage: regress invRate on all the exogenous+instruments
    firstStage_invRate_gam <- gam(log(mean_invRate) ~ s(LogGDPpwRel_initialYear) +s(log(mean_empGR_augm))+s(log(mean_hc_index))+ s(log(INV.RATE.1991)))
    summary(firstStage_invRate_gam)
    plot(firstStage_invRate_gam, page=1)
    
    #2) Second-stage: regress y on all the variables+residuals values first stage
	residuals_firstStage <- firstStage_invRate_gam$residuals
    conditionalConvergence_gam_endgInvRate_CF <- gam(c(averageGrowthRateSingleRegions) ~ s(LogGDPpwRel_initialYear) + s(log(mean_invRate))+ residuals_firstStage +s(log(mean_empGR_augm))+s(log(mean_hc_index)))
	summary(conditionalConvergence_gam_endgInvRate_CF)





